using System;
using System.Windows.Forms;

namespace ESRI.BAServer.SOAP
{
	/// <summary>
	/// This provider requests a new token in a dialog using HTTPS SSL connection without saving
	/// user credentials.
	/// </summary>
	/// <remarks>In comparison with <see cref="HTTPTokenProvider"/>, this class does not save user
	/// credentials in the private members. When a new token is requested, it shows a dialog and
	/// allows user to type request parameters for receiving a short-lived or long-lived token.</remarks>
	public class HTTPTokenProviderDialog : TokenProvider
	{
		string tokenServiceUrl;
		string clientId = string.Empty;
		int timeout = 60;
		int maxTimeout = 14400;
		IWin32Window parent;

		#region Constructor

		/// <summary>
		/// Initializes a new instance of the <see cref="HTTPTokenProviderDialog"/> class.
		/// </summary>
		/// <param name="parent">Parent window or null.</param>
		/// <param name="tokenServiceUrl">Token service URL.</param>
		public HTTPTokenProviderDialog(IWin32Window parent, string tokenServiceUrl)
		{
			this.parent = parent;
			this.tokenServiceUrl = tokenServiceUrl;
		}

		#endregion

		#region Public Properties

		/// <summary>
		/// Gets or sets the Client ID string.
		/// </summary>
		/// <remarks>The Client ID is the IP address of client computer or the URL of the web application
		/// that the client browser is using. The valid Client ID value starts from the
		/// "ip." string for the IP-address case and from the "ref." string for the Web Referrer URL case.
		/// The wrong value is ignored when the dialog is opened.
		/// The Client ID can be assigned outside or changed inside the dialog.</remarks>
		public string ClientId
		{
			get { return clientId; }
			set { clientId = value == null ? string.Empty : value; }
		}

		/// <summary>
		/// Gets or sets an expiration timeout when a long-lived token is requested.
		/// </summary>
		/// <remarks>This property is a nonnegative integer and specifies an expiration timeout in minutes.
		/// If its value is equal to zero, the ArcGIS Server timeout for the short-lived token is used.
		/// The default timeout is 60 minutes. 
		/// </remarks>
		public int Timeout
		{
			get { return timeout; }
			set { timeout = Math.Max(Math.Min(value, maxTimeout), 0); }
		}

		/// <summary>
		/// Gets or sets the maximum allowed timeout.
		/// </summary>
		/// <remarks>The default maximum allowed timeout is 14400 (10 days).</remarks>
		public int MaxTimeout
		{
			get { return maxTimeout; }
			set { maxTimeout = Math.Max(value, 1440); }
		}

		#endregion

		/// <summary>
		/// Creates a new token.
		/// </summary>
		/// <exception cref="OperationCanceledException">The operation is canceled by user.</exception>
		public override void UpdateToken()
		{
			DialogResult result;
			using (HTTPTokenProviderForm form = new HTTPTokenProviderForm(this, tokenServiceUrl))
			{
				if (parent != null)
					result = form.ShowDialog(parent);
				else
				{
					form.StartPosition = FormStartPosition.CenterScreen;
					result = form.ShowDialog();
				}
				Application.DoEvents();
			}

			if (result != DialogResult.OK)
				throw new OperationCanceledException("Token update operation was canceled by user.");
		}
	}
}
