using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Text;
using System.Windows.Forms;
using System.Net;

namespace ESRI.BAServer.SOAP
{
	partial class HTTPTokenProviderForm : Form
	{
		HTTPTokenProviderDialog dialog;
		bool allowValidate = true; // Allows validation of text in text boxes.

		// Request parameters (user name and password parameters are extracted from text boxes)
		string tokenServiceUrl;
		string clientIp = string.Empty;
		string clientUrl = string.Empty;
		int timeout = 0;

		// Error flags
		bool userNameError;
		bool passwordError;
		bool clientIdError = false;
		bool timeoutError = false;

		#region Constructor

		public HTTPTokenProviderForm(HTTPTokenProviderDialog dialog, string tokenServiceUrl)
		{
			InitializeComponent();
			this.dialog = dialog;
			this.tokenServiceUrl = tokenServiceUrl;
			PrepareErrorMessages();

			ValidateUserName(); // User name is empty here. Set error icon and message
			ValidatePassword(); // Password is also empty here. Set error icon and message

			// Parse initial client ID
			string id = dialog.ClientId;
			int idTypeIndex;
			if (string.IsNullOrEmpty(id)) idTypeIndex = 0; // No client ID
			else if (id.StartsWith("ip."))
			{
				idTypeIndex = 1; // Client IP type
				clientIp = id.Substring(3); // Client IP without prefix
			}
			else if (id.StartsWith("ref."))
			{
				idTypeIndex = 2; // Client URL type
				clientUrl = id.Substring(4); // Client URL without prefix
			}
			else idTypeIndex = 0; // Wrong client ID. Ignore it
			
			// Set the client ID type and value
			clientIdTypeComboBox.SelectedIndex = idTypeIndex;

			// Set the initial value for timeout
			timeout = dialog.Timeout;
			int unitsIndex = timeout == 0 ? 0 : timeout % 1440 == 0 ? 2 : timeout % 60 == 0 ? 1 : 0;

			// Set the timeout units and value
			unitsComboBox.SelectedIndex = unitsIndex;

			// Update the state of OK button
			UpdateOkButtonState();
		}

		void UpdateOkButtonState()
		{
			 okButton.Enabled = !(userNameError || passwordError || clientIdError || timeoutError);
		}

		#endregion

		#region Error handling

		string missingUserNameError;
		string missingPasswordError;
		string missingClientIdError;
		string wrongClientIpError;
		string wrongTimeoutError;
		string tooLargeTimeoutError;

		void PrepareErrorMessages()
		{
			// Error messages are stored in the invisible errorBox
			missingUserNameError = (string)errorBox.Items[0];
			missingPasswordError = (string)errorBox.Items[1];
			missingClientIdError = (string)errorBox.Items[2];
			wrongClientIpError = (string)errorBox.Items[3];
			wrongTimeoutError = (string)errorBox.Items[4];
			tooLargeTimeoutError = (string)errorBox.Items[5];
		}

		void ValidateUserName()
		{
			string message = userNameTextBox.Text.Trim().Length != 0 ? string.Empty : missingUserNameError;
			errorProvider.SetError(userNameTextBox, message);
			userNameError = message.Length != 0;
		}

		void ValidatePassword()
		{
			string message = passwordTextBox.Text.Trim().Length != 0 ? string.Empty : missingPasswordError;
			errorProvider.SetError(passwordTextBox, message);
			passwordError = message.Length != 0;
		}

		void ValidateClientId()
		{
			int index = clientIdTypeComboBox.SelectedIndex;
			string id = clientIdTextBox.Text.Trim();

			string message = index == 0 || id.Length != 0 ? string.Empty : missingClientIdError;
			clientIdError = message.Length != 0;

			if (!clientIdError)
				switch (index)
				{
				case 1:	// Validate client IP and save it in clientIp
					try
					{
						clientIp = System.Net.IPAddress.Parse(id).ToString();
					}
					catch (FormatException)
					{
						message = wrongClientIpError;
						clientIdError = true;
					}
					break;

				case 2: // Save id in clientUrl
					clientUrl = id;
					break;
				}
			errorProvider.SetError(clientIdTextBox, message);
		}

		void ValidateTimeout()
		{
			// Process the case when timeout is ignored
			if (clientIdTypeComboBox.SelectedIndex <= 0)
			{
				errorProvider.SetError(timeoutTextBox, string.Empty);
				timeoutError = false;
				return;
			}

			int index = unitsComboBox.SelectedIndex;
			int multiple = index <= 0 ? 1 : index == 1 ? 60 : 1440; // Get timeout units multiple
			int maxTimeout = dialog.MaxTimeout / multiple;          // Convert max timeout to units

			string message;
			try
			{
				string timeoutText = timeoutTextBox.Text.Trim();
				timeout = timeoutText.Length == 0 ? 0 : int.Parse(timeoutText);
				message = timeout < 0 ? wrongTimeoutError :
					timeout > maxTimeout ? tooLargeTimeoutError : string.Empty;
			}
			catch (Exception)
			{
				message = wrongTimeoutError;
			}
			errorProvider.SetError(timeoutTextBox, message);
			timeoutError = message.Length != 0;
			if (!timeoutError) timeout *= multiple; // Convert timeout to minutes
		}

		#endregion

		#region Events

		private void UserNameTextBox_TextChanged(object sender, EventArgs e)
		{
			if (allowValidate)
			{
				ValidateUserName();
				UpdateOkButtonState();
			}
		}

		private void PasswordTextBox_TextChanged(object sender, EventArgs e)
		{
			if (allowValidate)
			{
				ValidatePassword();
				UpdateOkButtonState();
			}
		}

		private void ClientIdTextBox_TextChanged(object sender, EventArgs e)
		{
			if (allowValidate)
			{
				ValidateClientId();
				UpdateOkButtonState();
			}
		}

		private void TimeoutTextBox_TextChanged(object sender, EventArgs e)
		{
			if (allowValidate)
			{
				ValidateTimeout();
				UpdateOkButtonState();
			}
		}

		private void ClientIdTypeComboBox_SelectedIndexChanged(object sender, EventArgs e)
		{
			allowValidate = false; // Ignore events on changes in text boxes
 
			// Update Client ID
			int index = clientIdTypeComboBox.SelectedIndex;
			clientIdTextBox.Text = index == 1 ? clientIp : index == 2 ? clientUrl : string.Empty;

			// Validate Client ID and timeout
			ValidateClientId();
			ValidateTimeout();

			// Update enabled state of controls used for requesting long-lived tokens
			bool longLivedTokenEnabled = index > 0;
			clientIdTextBox.Enabled = longLivedTokenEnabled;
			timeoutTextBox.Enabled = longLivedTokenEnabled;
			unitsComboBox.Enabled = longLivedTokenEnabled;

			UpdateOkButtonState();
			allowValidate = true;
		}

		private void UnitsComboBox_SelectedIndexChanged(object sender, EventArgs e)
		{
			allowValidate = false; // Ignore events on changes in text boxes

			int index = unitsComboBox.SelectedIndex;
			int multiple = index == 0 ? 1 : index == 1 ? 60 : 1440; // Get timeout units multiple
			int timeoutInUnits = (timeout + multiple / 2) / multiple; // Round to nearest value
			if (timeout != 0 && timeoutInUnits == 0) timeoutInUnits = 1; // Prevent zero value if timeout is positive

			timeoutTextBox.Text = timeoutInUnits == 0 ? string.Empty : timeoutInUnits.ToString();
			ValidateTimeout();

			UpdateOkButtonState();
			allowValidate = true;
		}

		private void OkButton_Click(object sender, EventArgs e)
		{
			int index = clientIdTypeComboBox.SelectedIndex;
			string query = "request=getToken";
			string clientId = string.Empty;

			// Prepare query parameters for long-lived token
			if (index > 0)
			{
				clientId = index == 1 ? "ip." + clientIp : "ref." + clientUrl;
				query += "&clientid=" + clientId;
				if (timeout != 0) query += "&expiration=" + timeout.ToString();
			}

			// Prepare common query parameters
			query += "&username=" + userNameTextBox.Text.Trim() +
				"&password=" + passwordTextBox.Text.Trim();

			// Get new token within the try-catch block
			string token;
			try
			{
				token = TokenProvider.GetToken(tokenServiceUrl, query);
			}
			catch (Exception ex)
			{
				if (ex.InnerException != null) ex = ex.InnerException;
				MessageBox.Show(this, ex.Message, "Token Error", MessageBoxButtons.OK, MessageBoxIcon.Error);
				return;
			}

			// Token was received successfully. Update dialog parameters
			dialog.CurrentToken = token;
			dialog.ClientId = clientId;
			if (clientId.Length != 0) dialog.Timeout = timeout;

			DialogResult = DialogResult.OK;
		}

		#endregion
	}
}