using System;
using System.Net;

namespace ESRI.BAServer.SOAP
{
	/// <summary>
	/// The base abstract implementation of the <see cref="ITokenProvider"/> interface.
	/// </summary>
	public abstract class TokenProvider : ITokenProvider
	{
		string token = null;
		static bool useTrustedSSLConnection = true;

		/// <summary>
		/// Initializes a new instance of the <see cref="TokenProvider"/> class.
		/// </summary>
		protected TokenProvider() { }

		#region ITokenProvider Members

		/// <summary>
		/// Gets or sets the current authentication token.
		/// </summary>
		/// <remarks>If the current token is null or empty, the first get operation applies
		/// the UpdateToken method.</remarks>
		public string CurrentToken
		{
			get
			{
				if (string.IsNullOrEmpty(token)) UpdateToken();
				return token;
			}
			set { token = value; }
		}

		/// <summary>
		/// Creates a new authentication token.
		/// </summary>
		public abstract void UpdateToken();

		#endregion

		#region Common Members

		/// <summary>
		/// Sets to use a trusted SSL connection or to allow a connection that is not trusted.
		/// </summary>
		/// <remarks>In the developer's environment, the SSL certificate is often "not trusted".
		/// False value of this property turns off the validation of the SSL certificate when
		/// a new token is requested. Default value is true.</remarks>
		public static bool UseTrustedSSLConnection { set { useTrustedSSLConnection = value; } }

		/// <summary>
		/// Gets a new token.
		/// </summary>
		/// <remarks>
		/// <para>The token service URL must use the Secure Hypertext Transfer Protocol (HTTPS)
		/// Secure Sockets Layer (SSL) connection. The query string contains the request type and
		/// user's authentication credentials, for example,
		/// "request=getToken&amp;username=...&amp;password=...".</para>
		/// <para>If the <c>UseTrustedSSLConnection</c> property is false, a non-trusted SSL certificate
		/// is allowed. 
		/// </para>
		/// </remarks>
		/// <param name="tokenServiceUrl">URL of the token service.</param>
		/// <param name="query">Query string.</param>
		/// <returns>The new token.</returns>
		public static string GetToken(string tokenServiceUrl, string query)
		{
			UriBuilder uriBuilder = new UriBuilder(tokenServiceUrl);
			uriBuilder.Query = query;
			System.Net.WebRequest request = System.Net.WebRequest.Create(uriBuilder.Uri);
			System.Net.WebResponse response;

			if (useTrustedSSLConnection)
			{
				response = request.GetResponse();
			}
			else
			{
				// Change the sertificate validation callback
				System.Net.Security.RemoteCertificateValidationCallback callback =
					ServicePointManager.ServerCertificateValidationCallback;
				ServicePointManager.ServerCertificateValidationCallback = NoCertificateValidationCallback;

				response = request.GetResponse();

				// Restore the sertificate validation callback
				ServicePointManager.ServerCertificateValidationCallback = callback;
			}

			return new System.IO.StreamReader(response.GetResponseStream()).ReadToEnd();
		}

		/// <summary>
		/// This callback method trusts any nonempty SSL certificate. 
		/// </summary>
		/// <param name="sender">An object that contains state information for this validation.</param>
		/// <param name="certificate">The certificate used to authenticate the remote party.</param>
		/// <param name="chain">The chain of certificate authorities associated with the remote certificate.</param>
		/// <param name="errors">One or more errors associated with the remote certificate.</param>
		/// <returns>True value.</returns>
		public static bool NoCertificateValidationCallback(object sender,
			System.Security.Cryptography.X509Certificates.X509Certificate certificate,
			System.Security.Cryptography.X509Certificates.X509Chain chain,
			System.Net.Security.SslPolicyErrors errors)
		{
			return certificate != null;
		}

		#endregion
	}
}
